/***************************************************************************
                          dcbrowsefiletree.cpp  -  description
                             -------------------
    begin                : Tue Feb 26 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qstring.h>
#include <qregexp.h>
#include <qlistview.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qfiledialog.h>
#include <qlineedit.h>

#include <dcconfig.h>
#include <dctransferview.h>
#include <dcmenuhandler.h>

#include "dcbrowsefiletree.h"

#include <dclib/dcos.h>
#include <dclib/cutils.h>

/** */
DCBrowseFileTree::DCBrowseFileTree(QWidget* parent, const char *name, int wflags,bool viewonly) : DCDialogBrowseFileTree(parent,name,wflags)
{
	int idx;

	// set the ColumnWidthMode to manual, we will take care of this task
	for( idx = 0; idx < ListView_FILETREE->columns(); idx++ )
	{
		ListView_FILETREE->setColumnWidthMode( idx, QListView::Manual );
	}

	ListView_FILETREE->setColumnAlignment(1,Qt::AlignRight);

	if( viewonly == FALSE )
	{
		InitDocument();
	}
}

DCBrowseFileTree::~DCBrowseFileTree()
{
}

/** overridden so that the columns are resized on show() */
void DCBrowseFileTree::show()
{
	QWidget::show();

	ResizeListViewColumn();
}

/** resize event handler */
void DCBrowseFileTree::resizeEvent( QResizeEvent * )
{
	ResizeListViewColumn();
}

/** resize the ListView columns */
void DCBrowseFileTree::ResizeListViewColumn()
{
	int width;

	if ( ListView_FILETREE->isVisible() )
	{
		width = ListView_FILETREE->width();

		ListView_FILETREE->setColumnWidth( 0, ((width*8)/10) );

		if ( ListView_FILETREE->verticalScrollBar()->isVisible() )
			ListView_FILETREE->setColumnWidth( 1, ((width*2)/10)-ListView_FILETREE->verticalScrollBar()->width()-4 );
		else
			ListView_FILETREE->setColumnWidth( 1, ((width*2)/10) );
	}
}

/** */
void DCBrowseFileTree::InitDocument()
{
	connect( ListView_FILETREE, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedFileList(QListViewItem*, const QPoint &, int )) );
	connect( ListView_FILETREE, SIGNAL(returnPressed(QListViewItem*)), this, SLOT(slotDoubleClickedFile(QListViewItem*)) );
	connect( ListView_FILETREE, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClickedFile(QListViewItem*)) );
}

/** */
void DCBrowseFileTree::InitTree( QString nick, QString hubname, QString hubhost, QString s )
{
	DC_QListViewItem * item, *item1;
	QString s1,sname,slength;
	long i,j,d;
	int depth;
	ulonglong filecount;
	ulonglong shared;

	sNick     = nick;
	sHubName  = hubname;
	sHubHost  = hubhost;
	sFileList = s;

	i = j = 0;
	filecount = 0;
	shared = 0;
	item = 0;

	ListView_FILETREE->setUpdatesEnabled(FALSE);

	while((i=s.find("\xD\xA",j))>0)
	{
		s1 = s.mid(j,i-j);

		depth=0;
		while( (s1.find("\x9",depth)) != -1 )
			depth++;
		s1 = s1.replace( QRegExp("\x9"), "" );

		if (item)
			while ( depth <= item->depth() )
				if ( (item=(DC_QListViewItem*)((QListViewItem*)item)->parent()) == 0 )
					break;

		if ( s1 != "" )
		{
			d = s1.findRev("|");

			if ( d != -1 ) // parse file entry
			{
				filecount++;

				sname   = s1.mid(0,d);
				slength = s1.right(s1.length()-d-1);

				if (item==0)
					item1 = new DC_QListViewItem( ListView_FILETREE );
				else
					item1 = new DC_QListViewItem( item );

				if ( slength == "" )
				{
					item1->myvalue = 0;
					slength = tr("<wrong length>");
				}
				else
				{
					item1->myvalue = CString(slength.ascii()).asULL();
					shared += CString(slength.ascii()).asULL();
				}

				item1->mycol = 1;
				item1->setText(0,sname);
				item1->setText(1,slength);
			}
			else // parse dir entry
			{
				if (item==0)
					item = new DC_QListViewItem( ListView_FILETREE );
				else
					item = new DC_QListViewItem( item );
				item->myvalue = 0;
				item->mycol   = 1;
				item->setText(0,s1);
			}
		}

		j = i+2;
	}

	LineEdit_FILES->setText( CString().setNum(filecount).Data() );
	LineEdit_SHARED->setText( CUtils::GetSizeString(shared,Config->GetUnit()).Data() );

	ListView_FILETREE->setUpdatesEnabled(TRUE);
	ListView_FILETREE->triggerUpdate();

	ResizeListViewColumn();
}

/** */
QString DCBrowseFileTree::CreateRemotePath( QListViewItem * item )
{
	QString s;
	QListViewItem * pitem;

	if(!item)
		return "";

	pitem = item;
	s = item->text(0);

	while ( (pitem=pitem->parent()) != 0 )
	{
		s = pitem->text(0) + "\\" + s;
	}

	return s;
}

/** */
void DCBrowseFileTree::slotRightButtonClickedFileList( QListViewItem * item, const QPoint &, int )
{
	// no nick for download
	if ( (sNick == "") || (item == 0) )
	{
		return;
	}

	FileTool.Init();

	QPopupMenu* m = new QPopupMenu(this);

	DCMenuHandler::InsertMenu(m, emiDOWNLOAD);
	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_TO);
	// disable 'download as' on directories
	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_AS, (item->text(1) == QString::null ? FALSE : TRUE));
	DCMenuHandler::InsertMenu(m, emiSEPARATOR);
	DCMenuHandler::InsertMenu(m, emiSAVE);

	int id = m->exec(QCursor::pos());

	delete m;

	if (id == -1)
	{
		return;
	}

	QString rootPath  = QString::null;
	QString localName = item->text(0);

	// Check if the user wants a non-default destination
	if (id == emiDOWNLOAD_TO)
	{
		rootPath = QFileDialog::getExistingDirectory("", this, "dldia",
                                                     tr("Select destination"));
		if ( rootPath.isEmpty() )
		{
			// If the user cancel, then we don't download
			return;
		}
	}
	// save filelist
	else if ( id == emiSAVE )
	{
		QString s = QFileDialog::getSaveFileName(
			"",
			"DCLST (*.DcLst)",
			this,
			"save file dialog",
			tr("Choose a filename to save under") );

		if ( s != "" )
		{
			QFile f(s);

			if ( f.open(IO_WriteOnly) == TRUE )
			{
				f.writeBlock(sFileList,sFileList.length());
				f.close();
			}
		}
	}
	// Should the file be renamed
	else if(id == emiDOWNLOAD_AS)
	{
		rootPath = QFileDialog::getSaveFileName("", "", this, "dldia",
                                                tr("Select a filename"));
		if ( rootPath.isEmpty() )
		{
			return;
		}

		QFileInfo fi(rootPath);
		rootPath = fi.dirPath();
		localName = fi.fileName();

		if ( rootPath.isEmpty() || localName.isEmpty() )
		{
			return;
		}
	}

	// Is this a download?
	if ( (id == emiDOWNLOAD) || (id == emiDOWNLOAD_TO) || (id == emiDOWNLOAD_AS) )
	{
		QString remotePath = CreateRemotePath(item);

		// Directory download
		if ( item->text(1).isEmpty() )
		{
			SendAllFiles(remotePath, item->text(0), rootPath, item);
		}
		// Single file download
		else
		{
			ulonglong size = CString(item->text(1).ascii()).asULL();

			FileTool.CheckFile(sNick.ascii(),
                                           sHubName.ascii(),
                                           sHubHost.ascii(),
                                           remotePath.ascii(),
                                           localName.ascii(), "",
                                           rootPath.ascii(), eltFILE,
                                           size );
		}
	}
}

/** */
void DCBrowseFileTree::SendAllFiles( QString remotepath, QString localpath, QString localrootpath, QListViewItem * item )
{
	QString s;
	QListViewItem * item1;

	if ( item == 0 )
	{
		return;
	}

	// no directory with files selected
	if ( item->childCount() == 0 )
	{
		return;
	}

	item1 = item->firstChild();

	while(item1)
	{
		if ( item1->text(1).isEmpty() )
		{
			// is a directory
			SendAllFiles( remotepath + "\\" + item1->text(0), localpath + "\\" + item1->text(0), localrootpath, item1 );
		}
		else
		{
			// is a file
			s = remotepath + "\\" + item1->text(0);

			// todo: check file ...
			FileTool.CheckFile( sNick.ascii(), sHubName.ascii(), sHubHost.ascii(),
							s.ascii(), item1->text(0).ascii(), localpath.ascii(), localrootpath.ascii(), eltFILE,
							CString(item1->text(1).ascii()).asULL() );
		}

		// next child
		item1 = item1->nextSibling();
	}
}

/** */
void DCBrowseFileTree::slotDoubleClickedFile(QListViewItem* item)
{
	ulonglong size;
	QString localName;
	QString remotePath;

	// no nick for download or nothing selected
	if ( (sNick == "") || (item == 0) )
	{
		return;
	}

	// only for files
	if ( item->text(1).isEmpty() )
	{
		return;
	}

	size       = CString(item->text(1).ascii()).asULL();
	localName  = item->text(0);
	remotePath = CreateRemotePath(item);

	// add file as new source
	FileTool.AddFileSource( sNick.ascii(),
				   sHubName.ascii(),
				   sHubHost.ascii(),
				   remotePath.ascii(),
				   localName.ascii(), "",
				   "", eltFILE,
				   size );

	return;
}
